library ieee;
use ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
use std.env.stop;
use std.textio.all;
use IEEE.numeric_std.all;
use ieee.std_logic_textio.all;

entity ciphertb is
end ciphertb;

architecture tb of ciphertb is

component cmprcipherv3
	port(clk, rst : in std_logic;
        key, IV : in std_logic_vector(83 downto 0); -- 84-bit key, 84-bit IV
		o_vld : out std_logic; -- Rises to high when cipher has finished initializing
		keystream_bit_out : out std_logic); -- Keystream (encryption) bit output
end component;



file file_RESULTS : text open write_mode is "output.txt";
file file_INPUT : text open read_mode is "input.txt";

signal count : integer := 1;
signal clk : std_logic := '0';
signal o_vld : std_logic := '0';
signal keystream_bit_out : std_logic;
signal keystream : std_logic_vector(19999 downto 0) := (others => '0');
signal rst : std_logic := '1'; -- reset
signal key : std_logic_vector(83 downto 0) := "011001010011001100001000010000010000011011111111101101011001101001111001111000011101";
signal IV : std_logic_vector(83 downto 0);

constant clk_period : time := 1 ns; -- Testbench clock period (arbitrary)

begin

uut : cmprcipherv3 port map(clk => clk, rst => rst, key => key, IV => IV, o_vld => o_vld, 
keystream_bit_out => keystream_bit_out);

clk_process : process
begin
    clk <= '0';
    wait for clk_period/2;
    clk <= '1';
    wait for clk_period/2;
end process;

stimulus : process
variable v_ILINE : line; -- input file line to read
variable v_OLINE : line; -- output file line to write
variable binary : std_logic_vector(83 downto 0); -- intermediate variable for reading from file

begin

   while (not(endfile(file_INPUT))) loop
   wait for clk_period;
   readline(file_INPUT, v_ILINE);
   bread(v_ILINE, binary);
   IV <= binary;
   wait for clk_period;
   rst <= '0';
   wait until o_vld = '1';
   wait for 2*clk_period;
   for i in 1 to 20000 loop
   wait for clk_period;
   keystream(i-1) <= keystream_bit_out;
   wait for clk_period;
   end loop;
   write(v_OLINE, keystream);
   writeline(file_RESULTS, v_OLINE);
   rst <= '1';
   keystream <= (others => '0');
   report "Keystreams generated: " & integer'image(count);
   count <= count + 1;
   end loop;
   if (endfile(file_INPUT)) then
   stop;
   end if;
end process;
end tb;
